import calendar
import json
import os
import re
from datetime import datetime
from enum import Enum

import holidays
import pandas as pd
import requests

date_format = "%d/%m/%Y"
days_of_week = {
    "Monday": 0,
    "Tuesday": 1,
    "Wednesday": 2,
    "Thursday": 3,
    "Friday": 4,
    "Saturday": 5,
    "Sunday": 6,
}


class Region(Enum):
    UK = 1
    England = 2
    Northern_Ireland = 3
    Scotland = 4
    Wales = 5


def check_postcode(postcode: str):
    """
    Checks a postcode exists and validates UK formatting against a RegEx string
        :param postcode: Postcode to parse
    """
    postcode_api_url = "https://api.postcodes.io/postcodes/"
    postcode_api_response = requests.get(f"{postcode_api_url}{postcode}")

    if postcode_api_response.status_code != 200:
        val_error = json.loads(postcode_api_response.text)
        raise ValueError(
            f"Exception: {val_error['error']} Status: {val_error['status']}"
        )
    return True


def check_paon(paon: str):
    """
    Checks that PAON data exists
        :param paon: PAON data to check, usually house number
    """
    try:
        if paon is None:
            raise ValueError("Invalid house number")
        return True
    except Exception as ex:
        print(f"Exception encountered: {ex}")
        print("Please check the provided house number.")
        exit(1)


def check_uprn(uprn: str):
    """
    Checks that the UPRN exists
        :param uprn: UPRN to check
    """
    try:
        if uprn is None or uprn == "":
            raise ValueError("Invalid UPRN")
        return True
    except Exception as ex:
        print(f"Exception encountered: {ex}")
        print("Please check the provided UPRN.")


def check_usrn(usrn: str):
    """
    Checks that the USRN exists
        :param uprn: USRN to check
    """
    try:
        if usrn is None or usrn == "":
            raise ValueError("Invalid USRN")
        return True
    except Exception as ex:
        print(f"Exception encountered: {ex}")
        print("Please check the provided USRN.")


def get_date_with_ordinal(date_number: int) -> str:
    """
    Return ordinal text on day of date
        :rtype: str
        :param date_number: Date number as an integer (e.g. 4)
        :return: Return date with ordinal suffix (e.g. 4th)
    """
    return str(date_number) + (
        "th"
        if 4 <= date_number % 100 <= 20
        else {1: "st", 2: "nd", 3: "rd"}.get(date_number % 10, "th")
    )


def remove_ordinal_indicator_from_date_string(date_string: str) -> str:
    """
    Remove the ordinal indicator from a written date as a string.
    E.g. June 12th 2022 -> June 12 2022
    :rtype: str
    """
    ord_day_pattern = re.compile(r"(?<=\d)(st|nd|rd|th)")
    return re.compile(ord_day_pattern).sub("", date_string)


def parse_header(raw_header: str) -> dict:
    """
    Parses a header string and returns one that can be useful
            :rtype: dict
            :param raw_header: header as a string, with values to separate as pipe (|)
            :return: header in a dictionary format that can be used in requests
    """
    header = dict()
    for line in raw_header.split("|"):
        if line.startswith(":"):
            a, b = line[1:].split(":", 1)
            a = f":{a}"
        else:
            a, b = line.split(":", 1)

        header[a.strip()] = b.strip()

    return header


def is_holiday(date_to_check: datetime, region: Region = Region.UK) -> bool:
    """
    Checks if a given date is a public holiday
        :param date_to_check: Date to check if holiday
        :param region: The UK nation to check. Defaults to UK.
        :return: Bool - true if a holiday, false if not
    """
    if region.name != "UK":
        subdiv = region.name.capitalize()
    else:
        subdiv = region.name

    uk_holidays = holidays.country_holidays("GB", subdiv=subdiv)

    if date_to_check in uk_holidays:
        return True
    else:
        return False


def get_weekday_dates_in_period(start: datetime, day_of_week: int, amount=8) -> list:
    """
    Returns a list of dates of a given weekday from a start date for the given amount of weeks
        :param start: Start date
        :param day_of_week: Day of week number. Recommended to use calendar.DAY (Monday=0, Sunday=6)
        :param amount: Number of weeks to get dates. Defaults to 8 weeks.
        :return: List of dates where the specified weekday is in the period
    """
    return (
        pd.date_range(
            start=start, freq=f"W-{calendar.day_abbr[day_of_week]}", periods=amount
        )
        .strftime(date_format)
        .tolist()
    )


def get_dates_every_x_days(start: datetime, x: int, amount: int = 8) -> list:
    """
    Returns a list of dates for `X` days from start date. For example, calling `get_stepped_dates_in_period(s, 21, 4)` would
    return `4` dates every `21` days from the start date `s`
        :param start: Date to start from
        :param x: X amount of days
        :param amount: Number of dates to find
        :return: List of dates every X days from start date
        :rtype: list
    """
    return (
        pd.date_range(start=start, freq=f"{x}D", periods=amount)
        .strftime(date_format)
        .tolist()
    )


def get_next_occurrence_from_day_month(date: datetime) -> datetime:
    day = datetime.now().date().strftime("%d")
    month = datetime.now().date().strftime("%m")
    if date.strftime("%m") < month or (date.strftime("%m") == month and date.strftime("%d") < day):
        date = pd.to_datetime(date) + pd.DateOffset(years=1)
    return date


def remove_alpha_characters(input_string: str) -> str:
    return "".join(c for c in input_string if c.isdigit() or c == " ")


def write_output_json(council: str, content: str):
    cwd = os.getcwd()
    outputs_path = os.path.join(cwd, "..", "tests", "outputs")
    if not os.path.exists(outputs_path) or not os.path.isdir(outputs_path):
        outputs_path = os.path.join(cwd, "uk_bin_collection", "tests", "outputs")
    if os.path.exists(outputs_path) and os.path.isdir(outputs_path):
        with open(os.path.join(outputs_path, council + ".json"), "w") as f:
            f.write(content)
    else:
        print("Exception encountered: Unable to save Output JSON file for the council.")
        print(
            "Please check you're running developer mode from either the UKBinCollectionData "
            "or uk_bin_collection/uk_bin_collection/ directories."
        )


def validate_dates(bin_dates: dict) -> dict:
    raise NotImplementedError()
    # If a date is in December and the next is in January, increase the year



import json
from collections import OrderedDict
from datetime import datetime

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass
import ssl
import urllib3

class CustomHttpAdapter (requests.adapters.HTTPAdapter):
    '''Transport adapter" that allows us to use custom ssl_context.'''

    def __init__(self, ssl_context=None, **kwargs):
        self.ssl_context = ssl_context
        super().__init__(**kwargs)

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = urllib3.poolmanager.PoolManager(
            num_pools=connections, maxsize=maxsize,
            block=block, ssl_context=self.ssl_context)


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object

        data = {"bins": []}
        uprn = kwargs.get("uprn")
        current_month = datetime.today().strftime("%m")
        current_year = datetime.today().strftime("%Y")
        url = (
            f"https://mybins.blackburn.gov.uk/api/mybins/getbincollectiondays?uprn={uprn}&month={current_month}"
            f"&year={current_year}"
        )

        # Build request header string, then parse it and get response
        response_header_str = (
            "Accept: application/json, text/plain, */*|Accept-Encoding: gzip, deflate, "
            "br|Accept-Language: en-GB,en;q=0.9|Connection: keep-alive|Host: "
            "mybins.blackburn.gov.uk|Referer: "
            "https://mybins.blackburn.gov.uk/calendar/MTAwMDEwNzUwNzQy|Sec-Fetch-Dest: "
            "empty|Sec-Fetch-Mode: cors|Sec-Fetch-Site: same-origin|Sec-GPC: 1|User-Agent: "
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/103.0.5060.134 Safari/537.36 "
        )
        response_headers = parse_header(response_header_str)
        requests.packages.urllib3.disable_warnings()
        session = requests.Session()
        ctx = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
        ctx.options |= 0x4
        session.mount('https://', CustomHttpAdapter(ctx))

        response = session.get(url, headers=response_headers)

        # Return JSON from response and loop through collections
        json_result = json.loads(response.content)
        bin_collections = json_result["BinCollectionDays"]
        for collection in bin_collections:
            if collection is not None:
                bin_type = collection[0].get("BinType")
                current_collection_date = datetime.strptime(
                    collection[0].get("CollectionDate"), "%Y-%m-%d"
                )
                next_collection_date = datetime.strptime(
                    collection[0].get("NextScheduledCollectionDate"), "%Y-%m-%d"
                )

                # Work out the most recent collection date to display
                if (
                        datetime.today().date()
                        <= current_collection_date.date()
                        < next_collection_date.date()
                ):
                    collection_date = current_collection_date
                else:
                    collection_date = next_collection_date

                dict_data = {
                    "type": bin_type,
                    "collectionDate": collection_date.strftime(date_format),
                }
                data["bins"].append(dict_data)

                data["bins"].sort(
                    key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
                )

        return data



import hashlib
import math
import time
from datetime import timedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


def ct(e: str) -> int:
    """
    Mimic ct() function in main.cbc0dd8a.js
        :rtype: int
        :param e: Day name
        :return: Day index
    """
    if e == "MON":
        return 0
    elif e == "TUE":
        return 1
    elif e == "WED":
        return 2
    elif e == "THU":
        return 3
    elif e == "FRI":
        return 4
    return -1


def ft() -> dict:
    """
    Mimic ft() function in main.cbc0dd8a.js
        :rtype: dict
        :return: Weeks and days
    """
    e = datetime.now()
    t = datetime(2022, 4, 20)
    return {
        "weeks": math.floor(((e - t).total_seconds() * 10) / 1e3 / 86400 / 7 % 2),
        "days": math.floor(((e - t).total_seconds() * 10) / 1e3 / 86400 % 7),
    }


def st(e: int, t: int, n: str) -> str:
    """
    Mimic st() function in main.cbc0dd8a.js
        :rtype: str
        :return: Date
    """
    d = datetime.now() + timedelta(days=(7 * t + (e - n)))
    return d.strftime(date_format)


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)",
            "origin": "https://kbccollectiveapi-coll-api.e4ff.pro-eu-west-1.openshiftapps.com",
            "referer": "https://kbccollectiveapi-coll-api.e4ff.pro-eu-west-1.openshiftapps.com/",
        }
        requests.packages.urllib3.disable_warnings()
        # Check council website workings haven't changed
        response = requests.get(
            f"https://kbccollectiveapi-coll-api.e4ff.pro-eu-west-1.openshiftapps.com/wc-info/static/js/main.cbc0dd8a.js",
            headers=headers,
        )
        if (
                response.status_code != 200
                or hashlib.sha256(response.text.encode("utf-8")).hexdigest()
                != "2f357c24b043c31c0157c234323c401238842c1d00f00f16c7ca3e569a0ab3cd"
        ):
            raise ValueError(
                "Council website has changed, parser needs updating. Please open issue on GitHub."
            )
        # Get variables for workings
        response = requests.get(
            f"https://api.northnorthants.gov.uk/test/wc-info/{uprn}?r={time.time() * 1000}",
            headers=headers,
        )
        if response.status_code != 200:
            raise ValueError("No bin data found for provided UPRN.")

        json_response = json.loads(response.text)
        sov = json_response["sov"]
        day = json_response["day"]
        schedule = json_response["schedule"]
        # Mimic workings in main.cbc0dd8a.js
        if sov == "ENC" or sov == "BCW":
            n = ft()
            r = 1
            if ct(day) > n["days"]:
                r = 0
            for o in range(0, 10):
                week = (n["weeks"] + o + r) % 2
                if (week == 0 and "B" == schedule) or (week != 0 and "B" != schedule):
                    bin_type = "General"
                else:
                    bin_type = "Recycling"
                collection_data = {
                    "type": bin_type,
                    "collectionDate": st(ct(day), o + r, n["days"]).replace(
                        ",", ""
                    ),
                }
                data["bins"].append(collection_data)

        return data



import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        url = "https://my.guildford.gov.uk/customers/s/sfsites/aura?r=15&other.BinScheduleDisplayCmp.GetBinSchedules=1"

        payload = 'message=%7B%22actions%22%3A%5B%7B%22id%22%3A%22250%3Ba%22%2C%22descriptor%22%3A%22apex%3A%2F' \
                  '%2FBinScheduleDisplayCmpController%2FACTION%24GetBinSchedules%22%2C%22callingDescriptor%22%3A' \
                  '%22markup%3A%2F%2Fc%3ABinScheduleDisplay%22%2C%22params%22%3A%7B%22database%22%3A%22domestic%22%2C' \
                  '%22UPRN%22%3A%22' + uprn + '%22%7D%2C%22version%22%3Anull%7D%5D%7D&aura.pageURI=%2Fcustomers%2Fs' \
                  '%2Fview-bin-collections&aura.context=%7B%22mode%22%3A%22PROD%22%2C%22fwuid%22%3A' \
                  '%22MlRqRU5YT3pjWFRNenJranFOMWFjQXlMaWFpdmxPSTZWeEo0bWtiN0hsaXcyNDQuMjAuNC0yLjQxLjQ%22%2C%22app%22' \
                  '%3A%22siteforce%3AcommunityApp%22%2C%22loaded%22%3A%7B%22APPLICATION%40markup%3A%2F%2Fsiteforce' \
                  '%3AcommunityApp%22%3A%22AHt2_xNq0mJPYC9ylIE4Ew%22%2C%22COMPONENT%40markup%3A%2F%2Finstrumentation' \
                  '%3Ao11ySecondaryLoader%22%3A%22WAlywPtXLxVWA9DxV-jd3A%22%2C%22COMPONENT%40markup%3A%2F' \
                  '%2Fflowruntime%3AflowRuntimeForFlexiPage%22%3A%22yd5ERlPoICEJEMf8W3eIXQ%22%7D%2C%22dn%22%3A%5B%5D' \
                  '%2C%22globals%22%3A%7B%7D%2C%22uad%22%3Afalse%7D&aura.token=null'

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Cookie': 'CookieConsentPolicy=0:1; LSKey-c$CookieConsentPolicy=0:1;'
        }

        response = requests.request("POST", url, headers=headers, data=payload)

        if response.status_code != 200:
            raise SystemError("Error retrieving data! Please try again or raise an issue on GitHub!")

        results = json.loads(response.text)

        if results["actions"][0]["state"] == "ERROR":
            raise ValueError("No collection data found for provided UPRN.")

        data = {"bins": []}

        schedules = results["actions"][0]["returnValue"]["FeatureSchedules"]

        for collection in schedules:
            bin_type = collection["FeatureName"]
            bin_collection = datetime.strptime(collection["NextDate"], "%Y-%m-%dT%H:%M:%S.%fZ")
            bin_previous_collection = datetime.strptime(collection["PreviousDate"], "%Y-%m-%dT%H:%M:%S.%fZ")
            if bin_collection:
                dict_data = {
                    "type": bin_type,
                    "collectionDate": bin_collection.strftime(date_format),
                    "previousCollectionDate": bin_previous_collection.strftime(date_format)
                }
                data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        # Get UPRN from kwargs
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        # Construct headers and set passed UPRN as a param for request
        headers = {
            'authority': 'www.rctcbc.gov.uk',
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'accept-language': 'en-GB,en;q=0.9',
            'cache-control': 'no-cache',
            # 'cookie': 'ASP.NET_SessionId=2b4gaaydt1rlu5pccgrpzamm',
            'pragma': 'no-cache',
            'referer': 'https://www.rctcbc.gov.uk/EN/Resident/RecyclingandWaste/RecyclingandWasteCollectionDays.aspx?PropertyNumber=&Postcode=CF72%209JD',
            'sec-ch-ua': '"Chromium";v="116", "Not)A;Brand";v="24", "Opera GX";v="102"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest': 'document',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-user': '?1',
            'upgrade-insecure-requests': '1',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.5845.188 Safari/537.36',
        }
        params = {
            'uprn': user_uprn,
        }

        response = requests.get(
            'https://www.rctcbc.gov.uk/EN/Resident/RecyclingandWaste/RecyclingandWasteCollectionDays.aspx',
            params=params,
            headers=headers,
        )

        # Throw an error if response is not HTTP200
        if response.status_code != 200:
            raise SystemError("Response status was not 200: Please raise an issue on GitHub!")

        # Parse response page and get all table rows
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()
        table_rows = soup.find("table", class_=("waste-table")).find_all("tr")

        # Find the bin type and parse the date for each row, then add to dict. First row will be just headings,
        # so skip it
        for row in table_rows:
            if len(row.contents) > 3:
                continue
            bin_type = row.contents[1].text.strip()
            bin_date_text = row.contents[2].text.split(',')[1].strip().replace(' of ', "")
            bin_date = datetime.strptime(remove_ordinal_indicator_from_date_string(bin_date_text),
                                         "%A %d %B %Y").strftime(date_format)

            dict_data = {
                "type": bin_type,
                "collectionDate": bin_date,
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    baseclass. They can also override some
    operations with a default implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        # Check the UPRN is valid
        check_uprn(uprn)

        # Request URL
        url = f"https://secure.sthelens.net/website/CollectionDates.nsf/servlet.xsp/NextCollections?source=1&refid={uprn}"

        # Make Request
        requests.packages.urllib3.disable_warnings()
        s = requests.session()
        page = s.get(url)

        # Make a BS4 object
        soup = BeautifulSoup(re.sub("<div([^>]+)>", "", page.text).replace("</div>", ""), features="html.parser")
        soup.prettify()

        data = {"bins": []}
        collection_rows = soup.find("table", {"class": "multitable"}).find("tbody").find_all("tr")

        for collection_row in collection_rows:
            # Get bin collection type
            bin_type = collection_row.find("th")
            if bin_type:
                bin_type = bin_type.get_text(strip=True)
                # Get bin collection dates
                for bin_date in collection_row.find_all("td"):
                    if bin_date.get_text(strip=True) != "Dates not allocated":
                        collection_date = datetime.strptime(bin_date.get_text(strip=True), "%a %d %b %Y")
                        dict_data = {
                            "type": bin_type,
                            "collectionDate": collection_date.strftime(date_format)
                        }
                        data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()
        # Define some months to get from the calendar
        this_month = datetime.now().month
        this_year = datetime.now().year
        one_month = this_month + 1
        two_month = this_month + 2
        months = [this_month, one_month, two_month]

        # Get and check the postcode and UPRN values
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        # Some data for the request
        cookies = {
            'ARRAffinity': 'c5a9db7fe43cef907f06528c3d34a997365656f757206fbdf34193e2c3b6f737',
            'ARRAffinitySameSite': 'c5a9db7fe43cef907f06528c3d34a997365656f757206fbdf34193e2c3b6f737',
        }
        headers = {
            'Accept': '*/*',
            'Accept-Language': 'en-GB,en;q=0.9',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            # 'Cookie': 'ARRAffinity=c5a9db7fe43cef907f06528c3d34a997365656f757206fbdf34193e2c3b6f737; ARRAffinitySameSite=c5a9db7fe43cef907f06528c3d34a997365656f757206fbdf34193e2c3b6f737',
            'Origin': 'https://ilambassadorformsprod.azurewebsites.net',
            'Pragma': 'no-cache',
            'Referer': 'https://ilambassadorformsprod.azurewebsites.net/wastecollectiondays/index',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-origin',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 OPR/98.0.0.0',
            'X-Requested-With': 'XMLHttpRequest',
            'sec-ch-ua': '"Chromium";v="112", "Not_A Brand";v="24", "Opera GX";v="98"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
        }

        collections = []

        # For each of the months we defined
        for cal_month in months:
            # If we're in Nov/Dec, the calculations won't work since its just adding one, so roll it
            # to next year correctly
            if cal_month == 13:
                cal_month = 1
                cal_year = this_year + 1
            elif cal_month == 14:
                cal_month = 2
                cal_year = this_year + 1
            else:
                cal_year = this_year

            # Data for the calendar
            data = {
                'Month': cal_month,
                'Year': cal_year,
                'Postcode': user_postcode,
                'Uprn': user_uprn,
            }

            # Send it all as a POST
            response = requests.post(
                'https://ilambassadorformsprod.azurewebsites.net/wastecollectiondays/collectionlist',
                cookies=cookies,
                headers=headers,
                data=data,
            )

            # If we don't get a HTTP200, throw an error
            if response.status_code != 200:
                raise SystemError("Error retrieving data! Please try again or raise an issue on GitHub!")

            soup = BeautifulSoup(response.text, features="html.parser")
            soup.prettify()

            # Find all the bits of the current calendar that contain an event
            events = soup.find_all("div", {"class": "rc-event-container"})

            for event in events:
                # Get the date and type of each bin collection
                bin_date = datetime.strptime(event.find_next("a").attrs.get("data-original-datetext"), "%A %d %B, %Y")
                bin_type = event.find_next("a").attrs.get("data-original-title")
                # Only process it if it's today or in the future
                if bin_date.date() >= datetime.now().date():
                    # Split the really long type up into two separate bins
                    if bin_type == 'Mixed dry recycling (blue lidded bin) and glass (black box or basket)':
                        collections.append(
                            ('Mixed dry recycling (blue lidded bin)', datetime.strftime(bin_date, date_format)))
                        collections.append(('Glass (black box or basket)', datetime.strftime(bin_date, date_format)))
                    else:
                        collections.append((bin_type, datetime.strftime(bin_date, date_format)))

        data = {"bins": []}

        # Now there's a list of collections, yeet them into the dictionary for nice JSON
        for item in collections:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1]
            }
            data["bins"].append(dict_data)

        return data



import pandas as pd
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import date_format
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    baseclass. They can also override some
    operations with a default implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get list items that can be seen on page
        for element in soup.find_all(
                "li", {"class": "list-group-item p-0 p-3 bin-collection-item"}
        ):
            element_text = element.text.strip().split("\n\n")
            element_text = [x.strip() for x in element_text]

            bin_type = element_text[1]
            collection_date = pd.Timestamp(element_text[0]).strftime(date_format)

            dict_data = {
                "type": bin_type,
                "collectionDate": collection_date,
            }
            data["bins"].append(dict_data)

        # Get hidden list items too
        for element in soup.find_all(
                "li", {"class": "list-group-item p-0 p-3 bin-collection-item d-none"}
        ):
            element_text = element.text.strip().split("\n\n")
            element_text = [x.strip() for x in element_text]

            bin_type = element_text[1]
            collection_date = pd.Timestamp(element_text[0]).strftime(date_format)

            dict_data = {
                "type": bin_type,
                "collectionDate": collection_date,
            }
            data["bins"].append(dict_data)

        return data
